# Written by Bram Cohen
# see LICENSE.txt for license information
# Updated and modified for ABC_OKC : Old King Cole

from threading import Event
from BitTornado.utility import getINIT_STATE


class DownloaderFeedback:
    def __init__(self, storagewrapper, choker, httpdl, sched, upfunc, downfunc,
                 extradataupfunc, leftfunc, file_length, finflag, sp,
                 statistics, config, statusfunc, interval):
        self.storagewrapper = storagewrapper
        self.choker = choker
        self.httpdl = httpdl
        self.sched = sched
        self.upfunc = upfunc
        self.downfunc = downfunc
        self.extradataupfunc = extradataupfunc
        self.leftfunc = leftfunc
        self.file_length = file_length
        self.finflag = finflag
        self.sp = sp
        self.statistics = statistics
        self.config = config
        self.lastids = []
        self.doneprocessing = Event()
        self.doneprocessing.set()
        self.statusfunc = statusfunc
        self.interval = interval
        self.httpseeddirection = getINIT_STATE()[1][0]
        self.autodisplay()

    def _rotate(self):
        cs = self.choker.connections
        for id in self.lastids:
            for i in xrange(len(cs)):
                if cs[i].get_id() == id:
                    return cs[i:] + cs[:i]
        return cs

    def spews(self):
        l = []
        totalspeed = 0.
        cs = self._rotate()
        self.lastids = [c.get_id() for c in cs]
        for c in cs:
            u = c.upload
            d = c.download
            a = {}
            a['dns'] = c.get_dns()
            a['optimistic'] = (c is self.choker.connections[0])
            a['direction'] = getINIT_STATE()[c.is_locally_initiated()][c.is_encrypted()]
            a['uprate'] = u.measure.get_rate()
            a['uinterested'] = u.is_interested()
            a['uchoked'] = u.is_choked()
            a['downrate'] = d.measure.get_rate()
            a['dinterested'] = d.is_interested()
            a['dchoked'] = d.is_choked()
            a['snubbed'] = d._check_snubbed()
            a['utotal'] = u.measure.get_total()
            a['dtotal'] = d.measure.get_total()
            if self.config['magnet']:
                a['completed'] = None
            elif d.gothavebitfield:
                if len(d.have) > 0:
                    a['completed'] = float(len(d.have) - d.have.nbzero) / len(d.have)
                else:
                    a['completed'] = 1.0
            else:
                a['completed'] = None
            if self.config['magnet']:
                a['speed'] = None
            elif d.have.complete():
                a['speed'] = 0.
            elif d.gothave:
                a['speed'] = d.peermeasure.get_lastrate()
                totalspeed += a['speed']
            else:
                a['speed'] = None

            l.append(a)                                               

        for dl in self.httpdl.get_downloads():
            if dl.goodseed:
                a = {}
                a['dns'] = (dl.baseurl, 80)
                a['optimistic'] = False
                a['direction'] = self.httpseeddirection
                a['uprate'] = 0
                a['uinterested'] = False
                a['uchoked'] = False
                a['downrate'] = dl.measure.get_rate()
                a['dinterested'] = True
                a['dchoked'] = a['snubbed'] = not dl.active
                a['utotal'] = None
                a['dtotal'] = dl.measure.get_total()
                a['completed'] = 1.0
                a['speed'] = None

                l.append(a)

        return l, totalspeed

    def gather(self):
        s = {}
        # Keep up and down and spew rate gathering at top (before statistics.update())
        # to get the more acurate values after the last updates of these measures
        s['up'] = self.upfunc()
        s['extradataup'] = self.extradataupfunc()
        if not self.finflag.isSet():
            s['down'] = self.downfunc()
        if self.sp.isSet():
            s['spew'], s['totspeed'] = self.spews()
        else:
            s['spew'] = None
        s['stats'] = self.statistics.update()

        if self.finflag.isSet():
            s['done'] = self.file_length
            return s

        obtained, desired = self.leftfunc()
        s['done'] = obtained
        s['wanted'] = desired
        if self.storagewrapper.initfrac is not None:
            s['frac'] = self.storagewrapper.initfrac
            s['ongoing'] = self.storagewrapper.initongoing
            s['act'] = self.storagewrapper.initact
        elif desired > 0:
            s['frac'] = float(obtained) / desired
            s['ongoing'] = None
            s['act'] = None
        elif self.config['magnet']:
            s['frac'] = 0.0
            s['ongoing'] = None
            s['act'] = None
        else:
            s['frac'] = 1.0
            s['ongoing'] = None
            s['act'] = None
        return s

    def autodisplay(self):
        self.sched(self.autodisplay, self.interval)
        if not self.doneprocessing.isSet():
            return
        self.doneprocessing.clear()
        stats = self.gather()
        if self.finflag.isSet():
            self.statusfunc(dpflag = self.doneprocessing, upRate = stats['up'],
                            extradataUpRate = stats['extradataup'],
                            statistics = stats['stats'], spew = stats['spew'],
                            totalspeed = stats['totspeed'])
        else:
            self.statusfunc(dpflag = self.doneprocessing,
                            fractionDone = stats['frac'], ongoing = stats['ongoing'],
                            sizeWanted = stats['wanted'], sizeDone = stats['done'],
                            downRate = stats['down'], upRate = stats['up'],
                            extradataUpRate = stats['extradataup'],
                            activity = stats['act'], statistics = stats['stats'],
                            spew = stats['spew'], totalspeed = stats['totspeed'])
