##############################################################################
# Module : bitfield.py
# Author : Old King Cole
# Date   : 07/11/2007
#
# Description : Bitfields
#
##############################################################################
class Bitfield:
    def __init__(self, nbbit = None, string = None, boollist = None, copyfrom = None):
        # Can be created with "nbbit" zero bits
        # or initialized with the "nbbit" first bits from a string "string"
        # or initialized from a list of booleans "boollist"
        # or duplicated from an existing Bitfield "copyfrom"
        if copyfrom is not None:
            self.nbbit = copyfrom.nbbit
            self.field = bytearray(copyfrom.field)
            self.nbzero = copyfrom.nbzero
        else:
            if boollist is not None:
                nbbit = len(boollist)
            elif nbbit is None:
                raise ValueError, "number of bits is mandatory"
            self.nbbit = nbbit
            nbchar, lastchar = divmod(nbbit, 8)
            if lastchar:
                nbchar += 1
            self.field = bytearray(nbchar)
            self.nbzero = nbbit
            if boollist is None:
                if string is None:
                    self.field = bytearray(nbchar)
                else:
                    if nbbit > len(string) << 3:
                        raise ValueError, "number of bits does not fit string length"
                    self.field = bytearray(string)
                    self.nbzero -= sum(self)
            else:
                for c in xrange(nbchar):
                    coord = c << 3
                    val = 0
                    for i in xrange(8):
                        boolnumber = coord + i
                        if boolnumber < nbbit and boollist[boolnumber]:
                            val += 1 << (7 - i)
                            self.nbzero -= 1
                    self.field[c] = val

    def getCoordinates(self, index):
        if index >= self.nbbit or index < 0:
            raise IndexError, "index out of range"
        return divmod(index, 8)

    def __setitem__(self, index, value):
        if index < 0:
            index = self.nbbit + index
        char, bit = self.getCoordinates(index)
        oldvalue = self.__getitem__(index)
        if value:
            if not oldvalue:
                self.field[char] = self.field[char] | 1 << (7 - bit)
                self.nbzero -= 1
        elif oldvalue:
            self.field[char] = self.field[char] & ~(1 << (7 - bit))
            self.nbzero += 1

    def __getitem__(self, index):
        if index < 0:
            index = self.nbbit + index
        char, bit = self.getCoordinates(index)
        return (self.field[char] >> (7 - bit)) & 1

    def __len__(self):
        return self.nbbit

    def __repr__(self):
        repr = ''
        for c in self.field:
            reprchar = ''
            for i in xrange(8):
                c, r = divmod(c, 2)
                reprchar = str(r) + reprchar
            repr += reprchar
        return repr[:self.nbbit]

    def tostring(self):
        return str(self.field)

    def complete(self):
        return not self.nbzero

    def empty(self):
        return self.nbzero == self.nbbit
